@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -132,17 +127,17 @@ class CApiTagHelper {
 	/**
 	 * Return SQL query conditions to filter host tags including inherited template tags.
 	 *
-	 * @static
-	 *
 	 * @param array  $tags
 	 * @param string $tags[]['tag']
 	 * @param int    $tags[]['operator']
 	 * @param string $tags[]['value']
 	 * @param int    $evaltype
+	 * @param string $parent_alias
 	 *
 	 * @return string
 	 */
-	public static function addInheritedHostTagsWhereCondition(array $tags, int $evaltype): string {
+	public static function addInheritedHostTagsWhereCondition(array $tags, int $evaltype,
+			string $parent_alias): string {
 		// Swap tag operators to select templates normally should be excluded.
 		$swapped_filter = array_map(function ($tag) {
 			$swapping_map = [
@@ -227,17 +222,13 @@ class CApiTagHelper {
 				foreach ($negated_tags[$tag_name] as $tag) {
 					$where['templateids'] += self::getMatchingTemplateids($tag, $db_template_tags);
 
-					if ($tag['operator'] == TAG_OPERATOR_NOT_EXISTS) {
-						$where['values'] = false;
-					}
-					elseif (is_array($where['values'])) {
-						$where['values'][] = self::makeTagWhereCondition($tag);
-					}
+					$where['values'][] = self::makeTagWhereCondition($tag);
 				}
 			}
 		});
 
-		$negated_where_conditions = [];
+		$where_conditions = [];
+
 		foreach ($negated_conditions as $tag => $tag_where) {
 
 			$templateids_in = [];
@@ -252,54 +243,39 @@ class CApiTagHelper {
 				]);
 			}
 
-			$negated_where_conditions[] = '(NOT EXISTS ('.
+			$where_conditions[] = '(NOT EXISTS ('.
 				'SELECT NULL'.
 				' FROM host_tag'.
-				' WHERE (h.hostid=host_tag.hostid'.
+				' WHERE ('.$parent_alias.'.hostid=host_tag.hostid'.
 					' AND host_tag.tag='.zbx_dbstr($tag).
-						($tag_where['values'] ? ' AND ('.implode(' OR ', $tag_where['values']).')' : '').
-					')'.
-					($templateids_in
-						? ' OR '.dbConditionInt('ht2.templateid', array_keys($templateids_in)).''
-						: ''
-					).
+					($tag_where['values'] ? ' AND ('.implode(' OR ', $tag_where['values']).')' : '').
 				')'.
+			')'.
+			($templateids_in
+				? ' AND NOT EXISTS ('.
+					'SELECT NULL'.
+					' FROM hosts_templates'.
+					' WHERE '.$parent_alias.'.hostid=hosts_templates.hostid'.
+						' AND '.dbConditionInt('hosts_templates.templateid', array_keys($templateids_in)).
+				')'
+				: '').
 			')';
 		}
 
-		$where_conditions = [];
-
-		if ($negated_where_conditions) {
-			if ($evaltype == TAG_EVAL_TYPE_AND_OR) {
-				$where_conditions[] = implode(' AND ', $negated_where_conditions);
-			}
-			else {
-				$where_conditions = array_map(function ($condition) {
-					return $condition;
-				}, $negated_where_conditions);
-			}
-		}
-
 		// Make 'where' conditions for inclusive filter tags.
 		foreach ($inclusive_tags as $tag_name => $tag_values) {
 			$templateids = [];
 			$values = [];
 
 			if ($tag_values === false) {
-				$templateids += self::getMatchingTemplateids(['tag' => $tag_name, 'operator' => TAG_OPERATOR_EXISTS],
-					$db_template_tags
-				);
+				$tag = ['tag' => $tag_name, 'operator' => TAG_OPERATOR_EXISTS];
+				$templateids += self::getMatchingTemplateids($tag, $db_template_tags);
 			}
 			else {
 				foreach ($tag_values as $tag) {
 					$templateids += self::getMatchingTemplateids($tag, $db_template_tags);
 
-					if ($tag['operator'] == TAG_OPERATOR_EXISTS) {
-						$values = false;
-					}
-					elseif (is_array($values)) {
-						$values[] = self::makeTagWhereCondition($tag);
-					}
+					$values[] = self::makeTagWhereCondition($tag);
 				}
 			}
 
@@ -318,11 +294,18 @@ class CApiTagHelper {
 			$where_conditions[] = '(EXISTS ('.
 				'SELECT NULL'.
 				' FROM host_tag'.
-				' WHERE h.hostid=host_tag.hostid'.
+				' WHERE '.$parent_alias.'.hostid=host_tag.hostid'.
 					' AND host_tag.tag='.zbx_dbstr($tag_name).
 					($values ? ' AND ('.implode(' OR ', $values).')' : '').
 				')'.
-				($templateids_in ? ' OR '.dbConditionInt('ht2.templateid', array_keys($templateids_in)) : '').
+				($templateids_in
+					? ' OR EXISTS ('.
+						'SELECT NULL'.
+						' FROM hosts_templates'.
+						' WHERE '.$parent_alias.'.hostid=hosts_templates.hostid'.
+							' AND '.dbConditionInt('hosts_templates.templateid', array_keys($templateids_in)).
+					')'
+					: '').
 			')';
 		}
 
